unit ErrorListUnit;

interface

uses
  SysUtils, Classes, Contnrs;

type
  TErrorList = class
  private
    FObjectList: TObjectList;
    function GetItem(Index: Integer): Exception;
  public
    constructor Create;
    destructor Destroy; override;

    function Add(Error: Exception): Integer;
    procedure Clear;
    function Count: Integer;
    function IndexOfClass(ErrorClass: TClass; StartIndex: Integer = 0): Integer;

    property Items[Index: Integer]: Exception read GetItem; default;
  end;

function ErrorList: TErrorList;

implementation

var
  uErrorList: TErrorList;

{ TErrorList }

function ErrorList: TErrorList;
begin
  if not Assigned(uErrorList) then
    uErrorList := TErrorList.Create;

  Result := uErrorList;
end;

constructor TErrorList.Create;
begin
  FObjectList := TObjectList.Create;
  FObjectList.OwnsObjects := True;
end;

destructor TErrorList.Destroy;
begin
  FObjectList.Free;
  inherited;
end;

function TErrorList.Add(Error: Exception): Integer;
begin
  Result := FObjectList.Add(Error);
end;

procedure TErrorList.Clear;
begin
  FObjectList.Clear;
end;

function TErrorList.Count: Integer;
begin
  Result := FObjectList.Count;
end;

function TErrorList.GetItem(Index: Integer): Exception;
begin
  Result := Exception(FObjectList[Index]);
end;

function TErrorList.IndexOfClass(ErrorClass: TClass; StartIndex: Integer = 0): Integer;
var
  I: Integer;
begin
  Result := -1;
  for I := StartIndex to FObjectList.Count - 1 do
  begin
    if FObjectList.Items[I] is ErrorClass then
    begin
      Result := I;
      break;
    end;
  end;
end;

initialization
  uErrorList := nil;

finalization
  if Assigned(uErrorList) then
    uErrorList.Free;

end.
